/****************************************************************
* Fondamenti di Informatica - Implementazione di un Buffer		*
* Struttura a Vettore - Metodi inserimento, estrazione			*
****************************************************************/

#include "stdio.h"
#include "stdlib.h"
#include "string.h"
#include "conio.h"
#include "float.h"
#include "math.h"

#define DIMBUF 5

class Buffer
{
	int ii; // indice di inserimento (posizione in cui inserire)
	int ie; // indice di estrazione (posizione dell'elemento da estrarre)
                // se ii==ie e' vuoto
	float corpo[DIMBUF];
public:
	Buffer();
	int inserimento(float a);  // inserisci un elemento
	int estrazione(float * a); // estrai un elemento 
	void svuota(void);  // svuota il buffer
	void stampa(void);  // stampa il buffer
	int getnumero(void); // numero di elementi
};

/********************************************************************/
Buffer::Buffer() 
  { 
	int i;   
	ii=ie=DIMBUF-1;  
	for (i=0;i<DIMBUF;i++) corpo[i]=0.0; 
  }

/********************************************************************
 Condizione di partenza per l'inserimento => ii=ie
 
 0 1							 N-1	
 _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _
| | | | | | | | | | | | | | | | | |
| | | | | | | | | | | | | | | | |A|	
|_|_|_|_|_|_|_|_|_|_|_|_|_|_|_|_|_|

							   ^ ^
						  <---ii ie

 0 1							 N-1	
 _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _
| | | | | | | | | | | | | | | | | |
| | | | | | | | | | | | | | | |B|A|	
|_|_|_|_|_|_|_|_|_|_|_|_|_|_|_|_|_|

							 ^   ^
						<---ii   ie

**********************************************************************
Condizione per il riaggiustamento
  
  -1 0 1 							 N-1	
	 _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _
	| | | | | | | | | | | | | | | | | |
	|S|R|Q|P|O|N|M|L|I|H|G|F| | | | | |	
	|_|_|_|_|_|_|_|_|_|_|_|_|_|_|_|_|_|

   ^					   ^
  ii<0                     ie

  
Al termine del riaggiustamento
  -1 0 1 							 N-1	
	 _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _
	| | | | | | | | | | | | | | | | | |
	| | | | | |S|R|Q|P|O|N|M|L|I|H|G|F|	
	|_|_|_|_|_|_|_|_|_|_|_|_|_|_|_|_|_|

			 ^					     ^
            ii                       ie
********************************************************************/

int Buffer::inserimento(float a)  // inserisci un elemento
{	
	int i;
	if (ii<0) 
	  { // necessita riorganizzazione se non e' pieno
		if (ie==DIMBUF-1) // pieno
			return(1);
	  // riorganizzazione
		for (i=ie;i>=0;i--)
			corpo[DIMBUF-1+i-ie]=corpo[i];  
	  // reimpostazione degli indici
		ii=DIMBUF-1-ie-1;
		ie=DIMBUF-1;
	  // cancellazione delle celle vecchie
		for (i=0;i<=ii;i++) corpo[i]=0.0;
	  }
	corpo[ii]=a;
	ii--;
	return(0);
}

/********************************************************************

 0 1							 N-1	
 _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _
| | | | | | | | | | | | | | | | | |
| | | | | | | | | | | | |E|D|C|B|A|	
|_|_|_|_|_|_|_|_|_|_|_|_|_|_|_|_|_|

					   ^         ^
					  ii         ie

 0 1							 N-1	
 _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _
| | | | | | | | | | | | | | | | | |
| | | | | | | | | | | | |E|D|C|B| |	--> A
|_|_|_|_|_|_|_|_|_|_|_|_|_|_|_|_|_|

					   ^       ^
					  ii   <---ie

Se ie==ii il buffer e' vuoto
********************************************************************/

int Buffer::estrazione(float * a) // estrai un elemento 
{
	if (ie>=0 && ii!=ie) 
	  { 
		*a=corpo[ie]; 
		corpo[ie]=0.0;
		ie--; 
		return(0); 
	  }
	return(1);
}

void Buffer::svuota(void)  // svuota buffer
  { 
	int i;   
	ii=ie=DIMBUF-1;  
	for (i=0;i<DIMBUF;i++) corpo[i]=0.0; 
  }

void Buffer::stampa(void)  // svuota buffer
{
	int i;
	for (i=0;i<DIMBUF;i++)
	  printf("%f, ",corpo[i]);
	printf("\n");
}

int Buffer::getnumero(void) // numero di elementi
  { return(ie-ii); }

/********************************************************************/

void main(void)
{	
	char c;
	int ret, flg=0;
	float val;
	Buffer buf;
	do {
	   printf("\nBuffer, operazioni: \n\n");
	   printf("a) inserisci \n");
	   printf("b) estrai \n");
	   printf("c) stampa \n");
	   printf("d) svuota \n");
	   printf("e) numero \n");
	   printf("q) esci \n\n"); 
	   printf("Premi un tasto: ");
	   c=getch();
	   printf("\n");
	   switch(c)
		  { 
			  case'a': case 'A':
				 printf("valore da inserire %f:"); scanf("%f",&val);
				 ret=buf.inserimento(val);
				 if (ret!=0) printf("Buffer pieno!\n"); 
				 break;
			  case 'b':  case 'B': 
				 ret=buf.estrazione(&val);
				 if (ret!=0) printf("Buffer vuoto!\n"); 
				 else printf("Valore estratto %f\n", val);
				 break;
			  case 'c':  case 'C': 
				 printf("Stampa \n");
				 buf.stampa();
				 break;
			  case 'd':  case 'D': 
				 printf("Svuota \n");
				 buf.svuota();
				 break;
			  case 'e':  case 'E': 
				 ret=buf.getnumero();
				 printf("Il buffer contiene %d elementi \n", ret);
				 break;
			  case 'q':  case 'Q': 
				 flg=1;
				 break; // uscita
			  default: printf("Operazione non consentita \n"); break;
		  }  
	   if (flg==0)
		  { printf("Premi un tasto per un'altra operazione\n"); 
		  getch(); 
		  }
	   } while (flg!=1);
}

/***************************************************************/




